# This file is part of VM-Experiments.
# Licensed under the AGPL-3.0-or-later. See LICENSE for details.

from flask import Flask, session, render_template, request, redirect, url_for
import sqlite3
import bcrypt

app = Flask(__name__)
app.secret_key = "your_secret_key"


def sqlite_prep():
    connection = sqlite3.connect("posts.db")
    cursor = connection.cursor()
    cursor.execute("""
    CREATE TABLE IF NOT EXISTS posts (
        id INTEGER PRIMARY KEY,
        title TEXT NOT NULL,
        user_id TEXT NOT NULL,
        description TEXT NOT NULL
    )
    """)
    cursor.execute("""
    CREATE TABLE IF NOT EXISTS users (
        id INTEGER PRIMARY KEY,
        username TEXT NOT NULL,
        password TEXT NOT NULL
    )
    """)


@app.route("/")
def index():
    sqlite_prep()
    connection = sqlite3.connect("posts.db")
    cursor = connection.cursor()
    cursor.execute("SELECT * FROM posts")
    posts = cursor.fetchall()
    if "user_id" not in session:
        status = "Log In"
    else:
        status = "Log Out"
    return render_template("index.html", posts=posts, status=status)


@app.route("/post")
def post():
    if "user_id" not in session:
        return redirect(url_for("account"))
    return render_template("post.html", status="Log Out")


@app.route("/account")
def account():
    if "user_id" not in session:
        return render_template("account.html", status="Log In")
    session.pop("user_id", None)
    return redirect(url_for("index"))


@app.route("/log_in", methods=["POST"])
def log_in():
    username = request.form["username"]
    password = request.form["password"]
    sqlite_prep()
    connection = sqlite3.connect("posts.db")
    cursor = connection.cursor()
    cursor.execute("SELECT * FROM users")
    users = cursor.fetchall()
    for user_id, db_username, db_password in users:
        if db_username == username:
            if bcrypt.checkpw(password.encode("utf-8"), db_password):
                session["user_id"] = user_id
                return redirect(url_for("index"))
            else:
                return "<p>WRONG PASSWORD</p>"
    salt = bcrypt.gensalt()
    hashed_password = bcrypt.hashpw(password.encode("utf-8"), salt)
    cursor.execute(
        "INSERT INTO users (username, password) VALUES (?, ?)",
        (username, hashed_password),
    )
    connection.commit()
    for user_id, db_username, db_password in users:
        if db_username == username:
            session["user_id"] = user_id
    return redirect(url_for("index"))


@app.route("/view", methods=["GET"])
def view():
    post = request.args.get("post")
    sqlite_prep()
    connection = sqlite3.connect("posts.db")
    cursor = connection.cursor()
    cursor.execute("SELECT * FROM posts")
    posts = cursor.fetchall()
    for post_id, title, user_id, description in posts:
        if post_id == int(post):
            if "user_id" not in session:
                status = "Log In"
            else:
                status = "Log Out"

            cursor.execute(
                """
                SELECT users.username 
                FROM posts 
                JOIN users ON posts.user_id = users.id 
                WHERE posts.id = ?
            """,
                (post_id,),
            )

            username = cursor.fetchone()
            if not username:
                username = ("Unknown",)
            return render_template(
                "view.html",
                post_id=post_id,
                user=username[0],
                title=title,
                description=description,
                status=status,
            )
    if "user_id" not in session:
        status = "Log In"
    else:
        status = "Log Out"
    return render_template("view.html", status=status)


@app.route("/create_post", methods=["POST"])
def create_post():
    if "user_id" not in session:
        return redirect(url_for("account"))
    title = request.form["title"]
    description = request.form["description"]
    sqlite_prep()
    connection = sqlite3.connect("posts.db")
    cursor = connection.cursor()
    cursor.execute(
        "INSERT INTO posts (title, user_id, description) VALUES (?, ?, ?)",
        (title, session["user_id"], description),
    )
    connection.commit()
    return redirect(url_for("index"))
